@@ -14,9 +14,7 @@ from verification.generators import registry as generators
 
 __all__ = [
     'VerificationError', 
-    'KeyManager',
     'KeyQuerySet',
-    'KeyMixin',
     'KeyGroup',
     'AbstractKey',
     'claim',
@@ -45,21 +43,21 @@ def claim(keystring, claimant):
     key_claimed.send_robust(sender=key, claimant=claimant, group=key.group)
     return key
 
-class KeyMixin(object):
+class KeyQuerySet(QuerySet):
 
     def expired(self):
         "Get keys that have expired"
         now = tznow()
-        return self.get_query_set().exclude(expires=None).filter(expires__lte=now)
+        return self.exclude(expires=None).filter(expires__lte=now)
 
     def available(self):
         "Get still available keys"
         now = tznow()
-        return self.get_query_set().filter(Q(expires__gt=now)|Q(expires=None)).filter(claimed=None)
+        return self.filter(Q(expires__gt=now)|Q(expires=None)).filter(claimed=None)
 
     def claimed(self):
         "Get claimed keys"
-        return self.get_query_set().exclude(claimed=None)
+        return self.exclude(claimed=None)
 
     def delete_expired(self):
         """Removes expired keys"""
@@ -69,13 +67,6 @@ class KeyMixin(object):
     def claim(self, *args):
         return claim(*args)
 
-class KeyQuerySet(QuerySet, KeyMixin):
-    pass
-
-class KeyManager(models.Manager, KeyMixin):
-
-    def get_query_set(self):
-        return KeyQuerySet(self.model, using=self._db)
 
 @python_2_unicode_compatible
 class KeyGroup(models.Model):
@@ -84,6 +75,7 @@ class KeyGroup(models.Model):
     ttl       - Time to live. A generated key will have "expires" set to
                 "pub_date" + "ttl", or will not expire of ttl is None
     generator - The name of a key generator
+    has_fact  - Whether Key.fact must be set
     """
     name = models.SlugField(max_length=32, primary_key=True)
     ttl = models.IntegerField('Time to live, in minutes', blank=True, null=True)
@@ -99,7 +91,7 @@ class KeyGroup(models.Model):
 
     def purge_keys(self):
         "Delete all keys belonging to this group"
-        model = self.key_set.model
+        model = self.keys.model
         model.objects.filter(group=self).delete()
 
     def generate_one_key(self, keycls, seed=None, fact=None, *args):
@@ -125,21 +117,26 @@ class AbstractKey(models.Model):
     claimed     - When the key was claimed
 
     Subclass this if keys are to be claimed by another model than AUTH_USER_MODEL.
-    
+
     Something like this:
 
         class YourSpecialKey(AbstractKey):
-            claimed_by = models.ForeignKey('yourapp.yourmodel', blank=True, null=True)
+            claimed_by = models.ForeignKey(
+                'yourapp.yourmodel',
+                on_delete=models.CASCADE,
+                blank=True,
+                null=True
+            )
 
-            objects = KeyManager()
+            objects = KeyQuerySet.as_manager()
 
     If you need to do more than just setting claimed_by to the correct
     yourapp.yourmodel-instance when claiming:
 
     1. Write your own claim()-function (it can call the existing claim()-function)
-    2. Subclass the KeyManager to call your claim-function::
+    2. Subclass the KeyQuerySet to call your claim-function::
 
-        class YourSpecialKeyManager(Keymanager):
+        class YourSpecialKeyQuerySet(KeyQuerySet):
 
             def claim(self, *args):
                 return yourclaim(*args)
@@ -147,9 +144,14 @@ class AbstractKey(models.Model):
     3. Replace the claim-method on the YourSpecialKey-class and add in your own manager::
 
         class YourSpecialKey(AbstractKey):
-            claimed_by = models.ForeignKey('yourapp.yourmodel', blank=True, null=True)
+            claimed_by = models.ForeignKey(
+                'yourapp.yourmodel',
+                on_delete=models.CASCADE,
+                blank=True,
+                null=True
+            )
 
-            objects = YourSpecialKeyManager()
+            objects = YourSpecialKeyQuerySet.as_manager()
 
             def claim(self, your_args):
                 return claim(self.key, yourargs)
@@ -157,13 +159,13 @@ class AbstractKey(models.Model):
     send_func = None
 
     key = models.CharField(unique=True, max_length=255)
-    group = models.ForeignKey(KeyGroup)
+    group = models.ForeignKey(KeyGroup, on_delete=models.CASCADE, related_name='keys')
     fact = models.TextField(blank=True, null=True)
-    pub_date = models.DateTimeField(auto_now_add=True)
+    pub_date = models.DateTimeField(default=tznow, editable=False, blank=True)
     expires = models.DateTimeField(blank=True, null=True)
     claimed = models.DateTimeField(blank=True, null=True)
 
-    objects = KeyManager()
+    objects = KeyQuerySet.as_manager()
 
     class Meta:
         abstract = True
@@ -179,11 +181,13 @@ class AbstractKey(models.Model):
 
     def save(self, *args, **kwargs):
         "Save key and set ttl if the group has it"
+        now = tznow()
+        if not self.pk:
+            self.pub_date = now
+            if self.group.ttl:
+                add_minutes = timedelta(minutes=self.group.ttl)
+                self.expires = self.pub_date + add_minutes
         super(AbstractKey, self).save(*args, **kwargs)
-        if self.group.ttl:
-            add_minutes = timedelta(minutes=self.group.ttl)
-            self.expires = self.pub_date + add_minutes
-            super(AbstractKey, self).save(*args, **kwargs)
 
     def clean(self):
         """Verify that facts is filled if the group demands it"""
@@ -209,6 +213,6 @@ class AbstractKey(models.Model):
 
 class Key(AbstractKey):
     """Standard key, claimable by AUTH_USER_MODEL"""
-    claimed_by = models.ForeignKey(settings.AUTH_USER_MODEL, blank=True, null=True, related_name='verification_keys')
+    claimed_by = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.CASCADE, blank=True, null=True, related_name='verification_keys')
 
-    objects = KeyManager()
+    objects = KeyQuerySet.as_manager()
