@@ -3,13 +3,15 @@ from __future__ import unicode_literals
 import logging
 _LOG = logging.getLogger(__name__)
 
-from django.http import HttpResponseRedirect, Http404
-from django.contrib.auth.decorators import login_required
-from django.contrib import messages
-from django.core.urlresolvers import reverse
+from django.http import HttpResponseRedirect
+try:
+    from django.urls import reverse
+except ImportError:   # Django < 1.9
+    from django.core.urlresolvers import reverse
+from django.core.exceptions import ImproperlyConfigured
 from django.views.generic.base import ContextMixin
 from django.views.generic import TemplateView, FormView, View
-from django.shortcuts import render, get_object_or_404
+from django.shortcuts import get_object_or_404
 
 from verification.models import Key, KeyGroup, VerificationError
 from verification.forms import LookupKeyForm
@@ -27,9 +29,8 @@ class KeyLookupMixin(object):
         self.key = get_object_or_404(self.model, key=key, group=group)
         return self.key
 
-
-class ClaimContextMixin(KeyLookupMixin, ContextMixin):
-    """Gets key and group from context:
+class ArgLookupMixin(object):
+    """Gets key and group from request:
     First checks url, then POST args, then GET args"""
 
     def get_key_arg(self):
@@ -54,6 +55,9 @@ class ClaimContextMixin(KeyLookupMixin, ContextMixin):
             urlgroup = self.keygroup
         return urlgroup
 
+class ClaimContextMixin(KeyLookupMixin, ArgLookupMixin, ContextMixin):
+    """Adds key and group to context"""
+
     def get_context_data(self, **kwargs):
         context = super(ClaimContextMixin, self).get_context_data(**kwargs)
         keyarg = self.get_key_arg()
@@ -108,7 +112,7 @@ class ClaimOnPostMixin(UrlClaimMixin):
 
 class ClaimOnGetView(ClaimOnGetMixin, View):
     """Claim a key for a logged-in user by visiting a "magic" url"""
-    http_method_names = ['get', 'head', 'options', 'trace'] 
+    http_method_names = ['get', 'head', 'options', 'trace']
 claim_get = ClaimOnGetView.as_view()
 
 class ClaimOnPostUrlView(ClaimOnPostMixin, TemplateView):
@@ -141,7 +145,7 @@ class AbstractClaimView(ClaimMixin, FormView):
             'key': keystring,
             'group': self.keygroup
         }
-        key = self.claim(keystring, self.keygroup)
+        self.claim(keystring, self.keygroup)
         return HttpResponseRedirect(self.get_success_url())
 
 class ClaimSuccessView(ClaimContextMixin, TemplateView):
